@@ -1,5 +1,6 @@
 #include "advanced-scene-switcher.hpp"
 #include "backup.hpp"
+#include "crash-handler.hpp"
 #include "log-helper.hpp"
 #include "macro-helpers.hpp"
 #include "obs-module-helper.hpp"
@@ -191,25 +192,21 @@ static void SaveSceneSwitcher(obs_data_t *save_data, bool saving, void *)
 
 		switcher->m.lock();
 		if (switcher->VersionChanged(data, g_GIT_SHA1)) {
-			auto json = obs_data_get_json(data);
-			static QString jsonQString = json ? json : "";
-			std::thread t([]() {
-				obs_queue_task(
-					OBS_TASK_UI,
-					[](void *) {
-						AskForBackup(jsonQString);
-					},
-					nullptr, false);
-			});
-			t.detach();
+			AskForBackup(data);
 		}
 
 		switcher->LoadSettings(data);
 		switcher->m.unlock();
 
-		if (!switcher->stop) {
-			switcher->Start();
+		if (switcher->stop) {
+			return;
 		}
+
+		if (ShouldSkipPluginStartOnUncleanShutdown()) {
+			return;
+		}
+
+		switcher->Start();
 	}
 }
 
@@ -421,7 +418,7 @@ bool SwitcherData::CheckForMatch(OBSWeakSource &scene,
 
 static void ResetMacros()
 {
-	for (auto &m : GetMacros()) {
+	for (auto &m : GetTopLevelMacros()) {
 		ResetMacroRunCount(m.get());
 		ResetMacroConditionTimers(m.get());
 	}
@@ -538,6 +535,7 @@ static void handleSceneChange()
 	}
 
 	switcher->checkDefaultSceneTransitions();
+	switcher->CheckAutoStart();
 }
 
 static void setLiveTime()
@@ -552,18 +550,28 @@ static void resetLiveTime()
 
 static void checkAutoStartRecording()
 {
+	if (switcher->obsIsShuttingDown) {
+		return;
+	}
+
 	if (switcher->autoStartEvent == SwitcherData::AutoStart::RECORDING ||
 	    switcher->autoStartEvent ==
-		    SwitcherData::AutoStart::RECORINDG_OR_STREAMING)
+		    SwitcherData::AutoStart::RECORINDG_OR_STREAMING) {
 		switcher->Start();
+	}
 }
 
 static void checkAutoStartStreaming()
 {
+	if (switcher->obsIsShuttingDown) {
+		return;
+	}
+
 	if (switcher->autoStartEvent == SwitcherData::AutoStart::STREAMING ||
 	    switcher->autoStartEvent ==
-		    SwitcherData::AutoStart::RECORINDG_OR_STREAMING)
+		    SwitcherData::AutoStart::RECORINDG_OR_STREAMING) {
 		switcher->Start();
+	}
 }
 
 static void handleTransitionEnd()
