@@ -3,6 +3,8 @@
 #include "filter-combo-box.hpp"
 #include "layout-helpers.hpp"
 #include "macro.hpp"
+#include "macro-search.hpp"
+#include "macro-settings.hpp"
 #include "path-helpers.hpp"
 #include "selection-helpers.hpp"
 #include "source-helpers.hpp"
@@ -25,15 +27,6 @@ void AdvSceneSwitcher::reject()
 	close();
 }
 
-void AdvSceneSwitcher::UpdateNonMatchingScene(const QString &name)
-{
-	OBSSourceAutoRelease scene =
-		obs_get_source_by_name(name.toUtf8().constData());
-	OBSWeakSourceAutoRelease ws = obs_source_get_weak_source(scene);
-
-	switcher->nonMatchingScene = ws;
-}
-
 void AdvSceneSwitcher::on_noMatchDontSwitch_clicked()
 {
 	if (loading) {
@@ -55,7 +48,6 @@ void AdvSceneSwitcher::on_noMatchSwitch_clicked()
 	std::lock_guard<std::mutex> lock(switcher->m);
 	switcher->switchIfNotMatching = NoMatchBehavior::SWITCH;
 	ui->noMatchSwitchScene->setEnabled(true);
-	UpdateNonMatchingScene(ui->noMatchSwitchScene->currentText());
 	ui->randomDisabledWarning->setVisible(true);
 }
 
@@ -131,17 +123,6 @@ void AdvSceneSwitcher::on_autoStartEvent_currentIndexChanged(int index)
 	switcher->autoStartEvent = static_cast<SwitcherData::AutoStart>(index);
 }
 
-void AdvSceneSwitcher::on_noMatchSwitchScene_currentTextChanged(
-	const QString &text)
-{
-	if (loading) {
-		return;
-	}
-
-	std::lock_guard<std::mutex> lock(switcher->m);
-	UpdateNonMatchingScene(text);
-}
-
 void AdvSceneSwitcher::on_checkInterval_valueChanged(int value)
 {
 	if (loading) {
@@ -205,16 +186,6 @@ void AdvSceneSwitcher::on_disableComboBoxFilter_stateChanged(int state)
 	FilterComboBox::SetFilterBehaviourEnabled(!state);
 }
 
-void AdvSceneSwitcher::on_disableMacroWidgetCache_stateChanged(int state)
-{
-	if (loading) {
-		return;
-	}
-
-	switcher->disableMacroWidgetCache = state;
-	MacroSegmentList::SetCachingEnabled(!state);
-}
-
 void AdvSceneSwitcher::on_warnPluginLoadFailure_stateChanged(int state)
 {
 	if (loading) {
@@ -451,16 +422,14 @@ void SwitcherData::LoadSettings(obs_data_t *obj)
 	}
 
 	// New post load steps to be declared during load
-	postLoadSteps.clear();
+	ClearPostLoadSteps();
 
 	// Needs to be loaded before any entries which might rely on scene group
 	// selections to be available.
 	loadSceneGroups(obj);
 	LoadVariables(obj);
 
-	for (const auto &func : loadSteps) {
-		func(obj);
-	}
+	RunLoadSteps(obj);
 
 	LoadMacros(obj);
 	LoadGlobalMacroSettings(obj);
@@ -481,7 +450,7 @@ void SwitcherData::LoadSettings(obs_data_t *obj)
 	LoadHotkeys(obj);
 	LoadUISettings(obj);
 
-	RunPostLoadSteps();
+	RunAndClearPostLoadSteps();
 
 	// Reset on startup and scene collection change
 	ResetLastOpenedTab();
@@ -516,18 +485,16 @@ void SwitcherData::SaveSettings(obs_data_t *obj)
 	SaveUISettings(obj);
 	SaveVersion(obj, g_GIT_SHA1);
 
-	for (const auto &func : saveSteps) {
-		func(obj);
-	}
+	RunSaveSteps(obj);
 }
 
 void SwitcherData::SaveGeneralSettings(obs_data_t *obj)
 {
 	obs_data_set_int(obj, "interval", interval);
 
-	std::string nonMatchingSceneName = GetWeakSourceName(nonMatchingScene);
-	obs_data_set_string(obj, "non_matching_scene",
-			    nonMatchingSceneName.c_str());
+	OBSDataAutoRelease noMatchScene = obs_data_create();
+	nonMatchingScene.Save(noMatchScene);
+	obs_data_set_obj(obj, "noMatchScene", noMatchScene);
 	obs_data_set_int(obj, "switch_if_not_matching",
 			 static_cast<int>(switchIfNotMatching));
 	noMatchDelay.Save(obj, "noMatchDelay");
@@ -540,8 +507,13 @@ void SwitcherData::SaveGeneralSettings(obs_data_t *obj)
 	obs_data_set_int(obj, "startup_behavior",
 			 static_cast<int>(startupBehavior));
 
-	obs_data_set_int(obj, "autoStartEvent",
-			 static_cast<int>(autoStartEvent));
+	OBSDataAutoRelease autoStart = obs_data_create();
+	obs_data_set_int(autoStart, "event", static_cast<int>(autoStartEvent));
+	obs_data_set_bool(autoStart, "useAutoStartScene", useAutoStartScene);
+	autoStartScene.Save(autoStart);
+	autoStartSceneName.Save(autoStart, "name");
+	autoStartSceneRegex.Save(autoStart);
+	obs_data_set_obj(obj, "autoStart", autoStart);
 
 	SaveLogLevel(obj);
 
@@ -550,8 +522,6 @@ void SwitcherData::SaveGeneralSettings(obs_data_t *obj)
 	obs_data_set_bool(obj, "disableHints", disableHints);
 	obs_data_set_bool(obj, "disableFilterComboboxFilter",
 			  disableFilterComboboxFilter);
-	obs_data_set_bool(obj, "disableMacroWidgetCache",
-			  disableMacroWidgetCache);
 	obs_data_set_bool(obj, "warnPluginLoadFailure", warnPluginLoadFailure);
 	obs_data_set_bool(obj, "hideLegacyTabs", hideLegacyTabs);
 
@@ -576,9 +546,14 @@ void SwitcherData::LoadGeneralSettings(obs_data_t *obj)
 				 static_cast<int>(NoMatchBehavior::NO_SWITCH));
 	switchIfNotMatching = static_cast<NoMatchBehavior>(
 		obs_data_get_int(obj, "switch_if_not_matching"));
-	std::string nonMatchingSceneName =
-		obs_data_get_string(obj, "non_matching_scene");
-	nonMatchingScene = GetWeakSourceByName(nonMatchingSceneName.c_str());
+
+	if (obs_data_has_user_value(obj, "noMatchScene")) {
+		OBSDataAutoRelease noMatchScene =
+			obs_data_get_obj(obj, "noMatchScene");
+		nonMatchingScene.Load(noMatchScene);
+	} else {
+		nonMatchingScene.Load(obj, "non_matching_scene");
+	}
 	noMatchDelay.Load(obj, "noMatchDelay");
 
 	cooldown.Load(obj, "cooldown");
@@ -599,8 +574,15 @@ void SwitcherData::LoadGeneralSettings(obs_data_t *obj)
 		stop = true;
 	}
 
-	autoStartEvent =
-		static_cast<AutoStart>(obs_data_get_int(obj, "autoStartEvent"));
+	OBSDataAutoRelease autoStart = obs_data_get_obj(obj, "autoStart");
+	autoStartEvent = static_cast<AutoStart>(
+		obs_data_has_user_value(obj, "autoStart")
+			? obs_data_get_int(autoStart, "event")
+			: obs_data_get_int(obj, "autoStartEvent"));
+	useAutoStartScene = obs_data_get_bool(autoStart, "useAutoStartScene");
+	autoStartScene.Load(autoStart);
+	autoStartSceneName.Load(autoStart, "name");
+	autoStartSceneRegex.Load(autoStart);
 
 	LoadLogLevel(obj);
 
@@ -609,8 +591,6 @@ void SwitcherData::LoadGeneralSettings(obs_data_t *obj)
 	disableHints = obs_data_get_bool(obj, "disableHints");
 	disableFilterComboboxFilter =
 		obs_data_get_bool(obj, "disableFilterComboboxFilter");
-	disableMacroWidgetCache =
-		obs_data_get_bool(obj, "disableMacroWidgetCache");
 	obs_data_set_default_bool(obj, "warnPluginLoadFailure", true);
 	warnPluginLoadFailure = obs_data_get_bool(obj, "warnPluginLoadFailure");
 	obs_data_set_default_bool(obj, "hideLegacyTabs", true);
@@ -680,10 +660,10 @@ void SwitcherData::CheckNoMatchSwitch(bool &match, OBSWeakSource &scene,
 		return;
 	}
 
-	if (switchIfNotMatching == NoMatchBehavior::SWITCH &&
-	    nonMatchingScene) {
+	auto noMatchScene = nonMatchingScene.GetScene(false);
+	if (switchIfNotMatching == NoMatchBehavior::SWITCH && noMatchScene) {
 		match = true;
-		scene = nonMatchingScene;
+		scene = noMatchScene;
 		transition = nullptr;
 	}
 	if (switchIfNotMatching == NoMatchBehavior::RANDOM_SWITCH) {
@@ -691,6 +671,27 @@ void SwitcherData::CheckNoMatchSwitch(bool &match, OBSWeakSource &scene,
 	}
 }
 
+void SwitcherData::CheckAutoStart()
+{
+	if (!useAutoStartScene) {
+		return;
+	}
+
+	bool shouldStartPlugin = false;
+	if (autoStartSceneRegex.Enabled()) {
+		const auto currentSceneName = GetWeakSourceName(currentScene);
+		shouldStartPlugin = autoStartSceneRegex.Matches(
+			currentSceneName, autoStartSceneName);
+	} else {
+		shouldStartPlugin = autoStartScene.GetScene(false) ==
+				    currentScene;
+	}
+
+	if (shouldStartPlugin) {
+		Start();
+	}
+}
+
 void SwitcherData::checkSwitchCooldown(bool &match)
 {
 	if (!match || !enableCooldown) {
@@ -833,7 +834,7 @@ static void setupGeneralTabInactiveWarning(QTabWidget *tabs)
 	inactiveTimer->start();
 }
 
-void advss::AdvSceneSwitcher::SetCheckIntervalTooLowVisibility() const
+void AdvSceneSwitcher::SetCheckIntervalTooLowVisibility() const
 {
 	auto macro = GetMacroWithInvalidConditionInterval();
 	if (!macro) {
@@ -857,8 +858,6 @@ void advss::AdvSceneSwitcher::SetCheckIntervalTooLowVisibility() const
 
 void AdvSceneSwitcher::SetupGeneralTab()
 {
-	PopulateSceneSelection(ui->noMatchSwitchScene, false);
-
 	if (switcher->switchIfNotMatching == NoMatchBehavior::SWITCH) {
 		ui->noMatchSwitch->setChecked(true);
 		ui->noMatchSwitchScene->setEnabled(true);
@@ -870,8 +869,17 @@ void AdvSceneSwitcher::SetupGeneralTab()
 		ui->noMatchRandomSwitch->setChecked(true);
 		ui->noMatchSwitchScene->setEnabled(false);
 	}
-	ui->noMatchSwitchScene->setCurrentText(
-		GetWeakSourceName(switcher->nonMatchingScene).c_str());
+	ui->noMatchSwitchScene->SetScene(switcher->nonMatchingScene);
+	ui->noMatchSwitchScene->LockToMainCanvas();
+
+	connect(ui->noMatchSwitchScene, &SceneSelectionWidget::SceneChanged,
+		this, [this](const SceneSelection &scene) {
+			if (loading) {
+				return;
+			}
+			std::lock_guard<std::mutex> lock(switcher->m);
+			switcher->nonMatchingScene = scene;
+		});
 
 	DurationSelection *noMatchDelay = new DurationSelection();
 	noMatchDelay->SetDuration(switcher->noMatchDelay);
@@ -906,9 +914,6 @@ void AdvSceneSwitcher::SetupGeneralTab()
 		switcher->disableFilterComboboxFilter);
 	FilterComboBox::SetFilterBehaviourEnabled(
 		!switcher->disableFilterComboboxFilter);
-	ui->disableMacroWidgetCache->setChecked(
-		switcher->disableMacroWidgetCache);
-	MacroSegmentList::SetCachingEnabled(!switcher->disableMacroWidgetCache);
 	ui->warnPluginLoadFailure->setChecked(switcher->warnPluginLoadFailure);
 	ui->hideLegacyTabs->setChecked(switcher->hideLegacyTabs);
 
@@ -922,6 +927,88 @@ void AdvSceneSwitcher::SetupGeneralTab()
 	populateAutoStartEventSelection(ui->autoStartEvent);
 	ui->autoStartEvent->setCurrentIndex(
 		static_cast<int>(switcher->autoStartEvent));
+	ui->autoStartSceneEnable->setChecked(switcher->useAutoStartScene);
+	ui->autoStartScene->SetScene(switcher->autoStartScene);
+	ui->autoStartScene->LockToMainCanvas();
+	ui->autoStartSceneName->setText(switcher->autoStartSceneName);
+	ui->autoStartSceneNameRegex->SetRegexConfig(
+		switcher->autoStartSceneRegex);
+
+	const auto setupAutoStartSceneLayoutVisibility = [this](bool useRegex) {
+		ui->autoStartSceneName->setVisible(useRegex);
+		ui->autoStartScene->setVisible(!useRegex);
+		if (useRegex) {
+			RemoveStretchIfPresent(ui->autoStartSceneLayout);
+		} else {
+			AddStretchIfNecessary(ui->autoStartSceneLayout);
+		}
+	};
+	setupAutoStartSceneLayoutVisibility(
+		switcher->autoStartSceneRegex.Enabled());
+
+	const auto setupAutoStartSceneWidgetState =
+		[this](bool useAutoStartScene) {
+			ui->autoStartScene->setEnabled(useAutoStartScene);
+			ui->autoStartSceneName->setEnabled(useAutoStartScene);
+			ui->autoStartSceneNameRegex->setEnabled(
+				useAutoStartScene);
+		};
+	setupAutoStartSceneWidgetState(switcher->useAutoStartScene);
+
+	connect(ui->autoStartSceneEnable, &QCheckBox::stateChanged, this,
+		[this, setupAutoStartSceneWidgetState](int enabled) {
+			if (loading) {
+				return;
+			}
+			std::lock_guard<std::mutex> lock(switcher->m);
+			switcher->useAutoStartScene = enabled;
+			setupAutoStartSceneWidgetState(enabled);
+		});
+
+	connect(ui->autoStartScene, &SceneSelectionWidget::SceneChanged, this,
+		[this](const SceneSelection &scene) {
+			if (loading) {
+				return;
+			}
+			std::lock_guard<std::mutex> lock(switcher->m);
+			switcher->autoStartScene = scene;
+			switcher->CheckAutoStart();
+		});
+	connect(ui->autoStartSceneName, &VariableLineEdit::editingFinished,
+		this, [this]() {
+			if (loading) {
+				return;
+			}
+			std::lock_guard<std::mutex> lock(switcher->m);
+			switcher->autoStartSceneName =
+				ui->autoStartSceneName->text().toStdString();
+			switcher->CheckAutoStart();
+		});
+	connect(ui->autoStartSceneNameRegex,
+		&RegexConfigWidget::RegexConfigChanged, this,
+		[this, setupAutoStartSceneLayoutVisibility](
+			const RegexConfig &regex) {
+			if (loading) {
+				return;
+			}
+			std::lock_guard<std::mutex> lock(switcher->m);
+			switcher->autoStartSceneRegex = regex;
+			setupAutoStartSceneLayoutVisibility(regex.Enabled());
+			switcher->CheckAutoStart();
+		});
+
+	ui->alwaysShowMacroSearch->setChecked(
+		GetMacroSearchSettings().showAlways);
+
+	connect(ui->alwaysShowMacroSearch, &QCheckBox::stateChanged, this,
+		[this](int enabled) {
+			GetMacroSearchSettings().showAlways = enabled;
+
+			if (loading) {
+				return;
+			}
+			CheckMacroSearchVisibility();
+		});
 
 	// Set up status control
 	auto statusControl = new StatusControl(this, true);
@@ -942,6 +1029,8 @@ void AdvSceneSwitcher::SetupGeneralTab()
 	setTabOrder(ui->importSettings, ui->cooldownTime);
 	setTabOrder(ui->cooldownTime, ui->noMatchDontSwitch);
 
+	SetupShowAllTabsCheckBox(ui->alwaysShowFeatureTabs, ui->tabWidget);
+
 	MinimizeSizeOfColumn(ui->statusLayout, 0);
 	setWindowTitle(windowTitle() + " - " + g_GIT_TAG);
 }
