@@ -1,6 +1,7 @@
 #include "macro-condition-macro.hpp"
 #include "layout-helpers.hpp"
 #include "macro-action-edit.hpp"
+#include "macro-signals.hpp"
 #include "macro.hpp"
 
 namespace advss {
@@ -26,6 +27,8 @@ const static std::map<MacroConditionMacro::Type, std::string>
 		 "AdvSceneSwitcher.condition.macro.type.actionEnabled"},
 		{MacroConditionMacro::Type::PAUSED,
 		 "AdvSceneSwitcher.condition.macro.type.paused"},
+		{MacroConditionMacro::Type::ACTIONS_PERFORMED,
+		 "AdvSceneSwitcher.condition.macro.type.actionsPerformed"},
 };
 
 const static std::map<MacroConditionMacro::CounterCondition, std::string>
@@ -99,7 +102,7 @@ bool MacroConditionMacro::CheckActionStateCondition()
 	if (!macro) {
 		return false;
 	}
-	if (!IsValidMacroSegmentIndex(macro.get(), _actionIndex - 1, false)) {
+	if (!IsValidActionIndex(macro.get(), _actionIndex - 1)) {
 		return false;
 	}
 	if (_type == Type::ACTION_DISABLED) {
@@ -121,6 +124,16 @@ bool MacroConditionMacro::CheckPauseState()
 	return macro->Paused();
 }
 
+bool MacroConditionMacro::CheckActionsPerformed()
+{
+	auto macro = _macro.GetMacro();
+	if (!macro) {
+		return false;
+	}
+
+	return macro->WasExecutedSince(macro->LastConditionCheckTime());
+}
+
 bool MacroConditionMacro::CheckCountCondition()
 {
 	auto macro = _macro.GetMacro();
@@ -158,6 +171,8 @@ bool MacroConditionMacro::CheckCondition()
 		return CheckActionStateCondition();
 	case Type::PAUSED:
 		return CheckPauseState();
+	case Type::ACTIONS_PERFORMED:
+		return CheckActionsPerformed();
 	default:
 		break;
 	}
@@ -300,6 +315,7 @@ MacroConditionMacroEdit::MacroConditionMacroEdit(
 	  _actionIndex(new MacroSegmentSelection(
 		  this, MacroSegmentSelection::Type::ACTION))
 {
+	_macros->HideGroups();
 	_count->setMaximum(10000000);
 	populateTypeSelection(_types);
 	populateCounterConditionSelection(_counterConditions);
@@ -307,7 +323,8 @@ MacroConditionMacroEdit::MacroConditionMacroEdit(
 
 	QWidget::connect(_macros, SIGNAL(currentTextChanged(const QString &)),
 			 this, SLOT(MacroChanged(const QString &)));
-	QWidget::connect(parent, SIGNAL(MacroRemoved(const QString &)), this,
+	QWidget::connect(MacroSignalManager::Instance(),
+			 SIGNAL(Remove(const QString &)), this,
 			 SLOT(MacroRemove(const QString &)));
 	QWidget::connect(_types, SIGNAL(currentIndexChanged(int)), this,
 			 SLOT(TypeChanged(int)));
@@ -414,6 +431,9 @@ void MacroConditionMacroEdit::SetupWidgets()
 	case MacroConditionMacro::Type::PAUSED:
 		SetupPauseWidgets();
 		break;
+	case MacroConditionMacro::Type::ACTIONS_PERFORMED:
+		SetupActionsPerformedWidgets();
+		break;
 	default:
 		break;
 	}
@@ -472,6 +492,14 @@ void MacroConditionMacroEdit::SetupPauseWidgets()
 		     _settingsLine1, {{"{{macros}}", _macros}});
 }
 
+void MacroConditionMacroEdit::SetupActionsPerformedWidgets()
+{
+	PlaceWidgets(
+		obs_module_text(
+			"AdvSceneSwitcher.condition.macro.actionsPerformed.entry"),
+		_settingsLine1, {{"{{macros}}", _macros}});
+}
+
 void MacroConditionMacroEdit::SetWidgetVisibility()
 {
 	_macros->setVisible(
@@ -481,7 +509,9 @@ void MacroConditionMacroEdit::SetWidgetVisibility()
 			MacroConditionMacro::Type::ACTION_DISABLED ||
 		_entryData->GetType() ==
 			MacroConditionMacro::Type::ACTION_ENABLED ||
-		_entryData->GetType() == MacroConditionMacro::Type::PAUSED);
+		_entryData->GetType() == MacroConditionMacro::Type::PAUSED ||
+		_entryData->GetType() ==
+			MacroConditionMacro::Type::ACTIONS_PERFORMED);
 	_counterConditions->setVisible(_entryData->GetType() ==
 				       MacroConditionMacro::Type::COUNT);
 	_count->setVisible(_entryData->GetType() ==
