@@ -7,6 +7,7 @@
 #include "macro-segment-copy-paste.hpp"
 #include "macro-segment-list.hpp"
 #include "macro-settings.hpp"
+#include "macro-signals.hpp"
 #include "math-helpers.hpp"
 #include "name-dialog.hpp"
 #include "path-helpers.hpp"
@@ -66,6 +67,8 @@ setupToolBar(const std::initializer_list<std::initializer_list<QWidget *>>
 MacroEdit::MacroEdit(QWidget *parent, QStringList helpMsg)
 	: ui(new Ui_MacroEdit)
 {
+	setParent(parent);
+
 	if (helpMsg.size() != 3) {
 		helpMsg << "AdvSceneSwitcher.macroTab.editConditionHelp"
 			<< "AdvSceneSwitcher.macroTab.editActionHelp"
@@ -106,6 +109,16 @@ MacroEdit::MacroEdit(QWidget *parent, QStringList helpMsg)
 	connect(ui->conditionsList, &QWidget::customContextMenuRequested, this,
 		&MacroEdit::ShowMacroConditionsContextMenu);
 
+	connect(this, &MacroEdit::MacroAdded, MacroSignalManager::Instance(),
+		&MacroSignalManager::Add);
+	connect(this, &MacroEdit::MacroRemoved, MacroSignalManager::Instance(),
+		&MacroSignalManager::Remove);
+	connect(this, &MacroEdit::MacroRenamed, MacroSignalManager::Instance(),
+		&MacroSignalManager::Rename);
+	connect(this, &MacroEdit::MacroSegmentOrderChanged,
+		MacroSignalManager::Instance(),
+		&MacroSignalManager::SegmentOrderChanged);
+
 	// Set action and condition toolbars
 	const std::string pathPrefix =
 		GetDataFilePath("res/images/" + GetThemeTypeName());
@@ -236,6 +249,11 @@ void MacroEdit::SetMacro(const std::shared_ptr<Macro> &macro)
 	}
 }
 
+std::shared_ptr<Macro> MacroEdit::GetMacro() const
+{
+	return _currentMacro;
+}
+
 void MacroEdit::ClearSegmentWidgetCacheFor(Macro *macro) const
 {
 	ui->conditionsList->ClearWidgetsFromCacheFor(macro);
@@ -400,8 +418,7 @@ void MacroEdit::PopulateMacroActions(Macro &m, uint32_t afterIdx)
 	ui->actionsList->SetVisibilityCheckEnable(false);
 
 	for (; afterIdx < actions.size(); afterIdx++) {
-		auto newEntry = new MacroActionEdit(this, &actions[afterIdx],
-						    actions[afterIdx]->GetId());
+		auto newEntry = new MacroActionEdit(this, &actions[afterIdx]);
 		ui->actionsList->Add(newEntry);
 	}
 
@@ -426,8 +443,7 @@ void MacroEdit::PopulateMacroElseActions(Macro &m, uint32_t afterIdx)
 	ui->elseActionsList->SetVisibilityCheckEnable(false);
 
 	for (; afterIdx < actions.size(); afterIdx++) {
-		auto newEntry = new MacroActionEdit(this, &actions[afterIdx],
-						    actions[afterIdx]->GetId());
+		auto newEntry = new MacroActionEdit(this, &actions[afterIdx]);
 		ui->elseActionsList->Add(newEntry);
 	}
 
@@ -454,8 +470,7 @@ void MacroEdit::PopulateMacroConditions(Macro &m, uint32_t afterIdx)
 
 	for (; afterIdx < conditions.size(); afterIdx++) {
 		auto newEntry = new MacroConditionEdit(
-			this, &conditions[afterIdx],
-			conditions[afterIdx]->GetId(), root);
+			this, &conditions[afterIdx], root);
 		ui->conditionsList->Add(newEntry);
 		root = false;
 	}
@@ -1062,11 +1077,10 @@ void MacroEdit::AddMacroAction(Macro *macro, int idx, const std::string &id,
 			macro->Actions().at(idx)->Load(data);
 		}
 		macro->Actions().at(idx)->PostLoad();
-		RunPostLoadSteps();
+		RunAndClearPostLoadSteps();
 		macro->UpdateActionIndices();
 		ui->actionsList->Insert(
-			idx,
-			new MacroActionEdit(this, &macro->Actions()[idx], id));
+			idx, new MacroActionEdit(this, &macro->Actions()[idx]));
 		SetActionData(*macro);
 	}
 	HighlightAction(idx);
@@ -1365,11 +1379,11 @@ void MacroEdit::AddMacroElseAction(Macro *macro, int idx, const std::string &id,
 			macro->ElseActions().at(idx)->Load(data);
 		}
 		macro->ElseActions().at(idx)->PostLoad();
-		RunPostLoadSteps();
+		RunAndClearPostLoadSteps();
 		macro->UpdateElseActionIndices();
 		ui->elseActionsList->Insert(
-			idx, new MacroActionEdit(
-				     this, &macro->ElseActions()[idx], id));
+			idx,
+			new MacroActionEdit(this, &macro->ElseActions()[idx]));
 		SetElseActionData(*macro);
 	}
 	HighlightElseAction(idx);
@@ -1565,13 +1579,13 @@ void MacroEdit::AddMacroCondition(Macro *macro, int idx, const std::string &id,
 			macro->Conditions().at(idx)->Load(data);
 		}
 		macro->Conditions().at(idx)->PostLoad();
-		RunPostLoadSteps();
+		RunAndClearPostLoadSteps();
 		(*cond)->SetLogicType(logic);
 		macro->UpdateConditionIndices();
 		ui->conditionsList->Insert(
 			idx,
 			new MacroConditionEdit(this, &macro->Conditions()[idx],
-					       id, idx == 0));
+					       idx == 0));
 		SetConditionData(*macro);
 	}
 	HighlightCondition(idx);
