@@ -4,6 +4,8 @@
 #include "macro-condition-edit.hpp"
 #include "macro-export-import-dialog.hpp"
 #include "macro-settings.hpp"
+#include "macro-search.hpp"
+#include "macro-signals.hpp"
 #include "macro-tree.hpp"
 #include "macro.hpp"
 #include "math-helpers.hpp"
@@ -23,6 +25,8 @@ namespace advss {
 
 static QObject *addPulse = nullptr;
 static QTimer onChangeHighlightTimer;
+static std::chrono::high_resolution_clock::time_point
+	lastOnChangeHighlightCheckTime{};
 
 static void disableAddButtonHighlight()
 {
@@ -115,7 +119,8 @@ void AdvSceneSwitcher::on_macroAdd_clicked()
 	auto selectedMacro = GetSelectedMacro();
 	if (!selectedMacro) {
 		ui->macros->Add(newMacro);
-		emit MacroAdded(QString::fromStdString(name));
+		MacroSignalManager::Instance()->Add(
+			QString::fromStdString(name));
 		return;
 	}
 
@@ -123,39 +128,28 @@ void AdvSceneSwitcher::on_macroAdd_clicked()
 		ui->macros->ExpandGroup(selectedMacro);
 		Macro::PrepareMoveToGroup(selectedMacro, newMacro);
 		ui->macros->AddToGroup(newMacro, selectedMacro);
-		emit MacroAdded(QString::fromStdString(name));
+		MacroSignalManager::Instance()->Add(
+			QString::fromStdString(name));
 		return;
 	}
 
 	auto selectedMacroGroup = selectedMacro->Parent();
 	if (!selectedMacroGroup) {
 		ui->macros->Add(newMacro, selectedMacro);
-		emit MacroAdded(QString::fromStdString(name));
+		MacroSignalManager::Instance()->Add(
+			QString::fromStdString(name));
 		return;
 	}
 
 	Macro::PrepareMoveToGroup(selectedMacroGroup, newMacro);
 	ui->macros->Add(newMacro, selectedMacro);
-	emit MacroAdded(QString::fromStdString(name));
+	MacroSignalManager::Instance()->Add(QString::fromStdString(name));
 }
 
 static void addGroupSubitems(std::vector<std::shared_ptr<Macro>> &macros,
 			     const std::shared_ptr<Macro> &group)
 {
-	std::vector<std::shared_ptr<Macro>> subitems;
-	subitems.reserve(group->GroupSize());
-
-	// Find all subitems
-	auto allMacros = GetMacros();
-	for (auto it = allMacros.begin(); it < allMacros.end(); it++) {
-		if ((*it)->Name() != group->Name()) {
-			continue;
-		}
-		for (uint32_t i = 1; i <= group->GroupSize(); i++) {
-			subitems.emplace_back(*std::next(it, i));
-		}
-		break;
-	}
+	auto subitems = GetGroupMacroEntries(group.get());
 
 	// Remove subitems which were already selected to avoid duplicates
 	for (const auto &subitem : subitems) {
@@ -203,9 +197,9 @@ void AdvSceneSwitcher::RemoveMacro(std::shared_ptr<Macro> &macro)
 	// Don't cache widgets for about to be deleted macros
 	MacroSegmentList::SetCachingEnabled(false);
 	ui->macros->Remove(macro);
-	MacroSegmentList::SetCachingEnabled(!switcher->disableMacroWidgetCache);
+	MacroSegmentList::SetCachingEnabled(true);
 
-	emit MacroRemoved(name);
+	MacroSignalManager::Instance()->Remove(name);
 }
 
 void AdvSceneSwitcher::RemoveSelectedMacros()
@@ -251,7 +245,8 @@ void AdvSceneSwitcher::RenameMacro(std::shared_ptr<Macro> &macro,
 		auto lock = LockContext();
 		macro->SetName(name.toStdString());
 	}
-	emit MacroRenamed(oldName, name);
+
+	MacroSignalManager::Instance()->Rename(oldName, name);
 }
 
 void AdvSceneSwitcher::on_macroRemove_clicked()
@@ -405,13 +400,17 @@ void AdvSceneSwitcher::ImportMacros()
 	int groupSize = 0;
 	std::shared_ptr<Macro> group;
 	std::vector<std::shared_ptr<Macro>> importedMacros;
+	auto &tempMacros = GetTemporaryMacros();
 
 	auto lock = LockContext();
 	for (size_t i = 0; i < count; i++) {
+		tempMacros.clear();
+
 		OBSDataAutoRelease array_obj = obs_data_array_item(array, i);
 		auto macro = std::make_shared<Macro>();
+		tempMacros.emplace_back(macro);
 		macro->Load(array_obj);
-		RunPostLoadSteps();
+		RunAndClearPostLoadSteps();
 
 		if (macroNameExists(macro->Name()) &&
 		    !ResolveMacroImportNameConflict(macro)) {
@@ -420,7 +419,7 @@ void AdvSceneSwitcher::ImportMacros()
 		}
 
 		importedMacros.emplace_back(macro);
-		GetMacros().emplace_back(macro);
+		GetTopLevelMacros().emplace_back(macro);
 		if (groupSize > 0 && !macro->IsGroup()) {
 			Macro::PrepareMoveToGroup(group, macro);
 			groupSize--;
@@ -439,9 +438,10 @@ void AdvSceneSwitcher::ImportMacros()
 	for (const auto &macro : importedMacros) {
 		macro->PostLoad();
 	}
-	RunPostLoadSteps();
+	RunAndClearPostLoadSteps();
+	tempMacros.clear();
 
-	ui->macros->Reset(GetMacros(),
+	ui->macros->Reset(GetTopLevelMacros(),
 			  GetGlobalMacroSettings()._highlightExecuted);
 	disableAddButtonHighlight();
 }
@@ -552,10 +552,14 @@ void AdvSceneSwitcher::HighlightOnChange() const
 		return;
 	}
 
-	if (macro->OnChangePreventedActionsRecently()) {
+	if (macro->OnChangePreventedActionsSince(
+		    lastOnChangeHighlightCheckTime)) {
 		HighlightWidget(ui->runMacroOnChange, Qt::yellow,
 				Qt::transparent, true);
 	}
+
+	lastOnChangeHighlightCheckTime =
+		std::chrono::high_resolution_clock::now();
 }
 
 void AdvSceneSwitcher::on_macroSettings_clicked()
@@ -568,7 +572,8 @@ void AdvSceneSwitcher::on_macroSettings_clicked()
 	}
 
 	GetGlobalMacroSettings() = prop;
-	emit HighlightMacrosChanged(prop._highlightExecuted);
+	MacroSignalManager::Instance()->HighlightChanged(
+		prop._highlightExecuted);
 
 	// Reset highlights to avoid all macro segments being highlighted, which
 	// would have been highlighted at least once since the moment the
@@ -604,17 +609,20 @@ setupToolBar(const std::initializer_list<std::initializer_list<QWidget *>>
 	auto toolbar = new QToolBar();
 	toolbar->setIconSize({16, 16});
 
-	QAction *lastSeperator = nullptr;
+	QAction *lastSeparator = nullptr;
 
 	for (const auto &widgetGroup : widgetGroups) {
 		for (const auto &widget : widgetGroup) {
+			if (!widget) {
+				continue;
+			}
 			toolbar->addWidget(widget);
 		}
-		lastSeperator = toolbar->addSeparator();
+		lastSeparator = toolbar->addSeparator();
 	}
 
-	if (lastSeperator) {
-		toolbar->removeAction(lastSeperator);
+	if (lastSeparator) {
+		toolbar->removeAction(lastSeparator);
 	}
 
 	// Prevent "extension" button from showing up
@@ -627,7 +635,9 @@ void AdvSceneSwitcher::SetupMacroTab()
 {
 	ui->macros->installEventFilter(this);
 
-	if (GetMacros().size() == 0 && !switcher->disableHints) {
+	auto &macros = GetTopLevelMacros();
+
+	if (macros.size() == 0 && !switcher->disableHints) {
 		addPulse = HighlightWidget(ui->macroAdd, QColor(Qt::green));
 	}
 
@@ -635,8 +645,7 @@ void AdvSceneSwitcher::SetupMacroTab()
 					   {ui->macroUp, ui->macroDown}});
 	ui->macroControlLayout->addWidget(macroControls);
 
-	ui->macros->Reset(GetMacros(),
-			  GetGlobalMacroSettings()._highlightExecuted);
+	ui->macros->Reset(macros, GetGlobalMacroSettings()._highlightExecuted);
 	connect(ui->macros, SIGNAL(MacroSelectionChanged()), this,
 		SLOT(MacroSelectionChanged()));
 	ui->runMacro->SetMacroTree(ui->macros);
@@ -649,6 +658,8 @@ void AdvSceneSwitcher::SetupMacroTab()
 	ui->macroPriorityWarning->setVisible(
 		switcher->functionNamesByPriority[0] != macro_func);
 
+	lastOnChangeHighlightCheckTime =
+		std::chrono::high_resolution_clock::now();
 	onChangeHighlightTimer.setInterval(1500);
 	connect(&onChangeHighlightTimer, SIGNAL(timeout()), this,
 		SLOT(HighlightOnChange()));
@@ -665,6 +676,12 @@ void AdvSceneSwitcher::SetupMacroTab()
 				switcher->macroListMacroEditSplitterPosition);
 		}
 	}
+
+	SetupMacroSearchWidgets(ui->macroSearchLayout, ui->macroSearchText,
+				ui->macroSearchClear, ui->macroSearchType,
+				ui->macroSearchRegex,
+				ui->macroSearchShowSettings,
+				[this]() { ui->macros->RefreshFilter(); });
 }
 
 void AdvSceneSwitcher::ShowMacroContextMenu(const QPoint &pos)
@@ -746,12 +763,12 @@ void AdvSceneSwitcher::CopyMacro()
 	newMacro->Load(data);
 	newMacro->PostLoad();
 	newMacro->SetName(name);
-	RunPostLoadSteps();
+	RunAndClearPostLoadSteps();
 	Macro::PrepareMoveToGroup(macro->Parent(), newMacro);
 
 	ui->macros->Add(newMacro, macro);
 	disableAddButtonHighlight();
-	emit MacroAdded(QString::fromStdString(name));
+	MacroSignalManager::Instance()->Add(QString::fromStdString(name));
 }
 
 bool MacroTabIsInFocus()
