@@ -1,7 +1,6 @@
 #include "macro.hpp"
 #include "macro-action-factory.hpp"
 #include "macro-condition-factory.hpp"
-#include "macro-dock.hpp"
 #include "macro-helpers.hpp"
 #include "macro-settings.hpp"
 #include "plugin-state-helpers.hpp"
@@ -15,69 +14,10 @@
 #include <QAction>
 #include <QMainWindow>
 #include <unordered_map>
-#include <util/platform.h>
-
-#if LIBOBS_API_VER < MAKE_SEMANTIC_VERSION(30, 0, 0)
-#include <QDockWidget>
-
-namespace {
-
-struct DockMapEntry {
-	QAction *action = nullptr;
-	QWidget *dock = nullptr;
-};
-
-} // namespace
-
-static std::unordered_map<const char *, DockMapEntry> dockIds;
-static std::mutex dockMutex;
-
-static bool obs_frontend_add_dock_by_id(const char *id, const char *title,
-					QWidget *widget)
-{
-	std::lock_guard<std::mutex> lock(dockMutex);
-	if (dockIds.count(id) > 0) {
-		return false;
-	}
-
-	widget->setObjectName(id);
-
-	auto dock = new QDockWidget();
-	dock->setWindowTitle(title);
-	dock->setWidget(widget);
-	dock->setFloating(true);
-	dock->setVisible(false);
-	dock->setFeatures(QDockWidget::DockWidgetClosable |
-			  QDockWidget::DockWidgetMovable |
-			  QDockWidget::DockWidgetFloatable);
-
-	auto action = static_cast<QAction *>(obs_frontend_add_dock(dock));
-	if (!action) {
-		return false;
-	}
-	dockIds[id] = {action, dock};
-	return true;
-}
-
-static void obs_frontend_remove_dock(const char *id)
-{
-	std::lock_guard<std::mutex> lock(dockMutex);
-	auto it = dockIds.find(id);
-	if (it == dockIds.end()) {
-		return;
-	}
-	it->second.action->deleteLater();
-	it->second.dock->deleteLater();
-	dockIds.erase(it);
-}
-
-#endif
 
 namespace advss {
 
-static std::deque<std::shared_ptr<Macro>> macros;
-
-Macro::Macro(const std::string &name)
+Macro::Macro(const std::string &name) : _dockSettings(this)
 {
 	SetName(name);
 }
@@ -99,12 +39,6 @@ Macro::~Macro()
 	_die = true;
 	Stop();
 	ClearHotkeys();
-
-	// Keep the dock widgets in case of shutdown so they can be restored by
-	// OBS on startup
-	if (!OBSIsShuttingDown()) {
-		RemoveDock();
-	}
 }
 
 std::shared_ptr<Macro>
@@ -123,6 +57,7 @@ Macro::CreateGroup(const std::string &name,
 
 void Macro::RemoveGroup(std::shared_ptr<Macro> group)
 {
+	auto &macros = GetTopLevelMacros();
 	auto it = std::find(macros.begin(), macros.end(), group);
 	if (it == macros.end()) {
 		return;
@@ -139,6 +74,7 @@ void Macro::RemoveGroup(std::shared_ptr<Macro> group)
 
 void Macro::PrepareMoveToGroup(Macro *group, std::shared_ptr<Macro> item)
 {
+	auto &macros = GetTopLevelMacros();
 	for (const auto &m : macros) {
 		if (m.get() == group) {
 			PrepareMoveToGroup(m, item);
@@ -306,8 +242,12 @@ bool Macro::CheckConditions(bool ignorePause)
 	vblog(LOG_INFO, "Macro %s returned %d", _name.c_str(), _matched);
 
 	_conditionSateChanged = _lastMatched != _matched;
-	if (!_conditionSateChanged && _performActionsOnChange) {
-		_onPreventedActionExecution = true;
+	const bool hasActionsToExecute = _matched ? (_actions.size() > 0)
+						  : (_elseActions.size() > 0);
+	if (!_conditionSateChanged && _performActionsOnChange &&
+	    hasActionsToExecute) {
+		_lastOnChangeActionsPreventedTime =
+			std::chrono::high_resolution_clock::now();
 	}
 
 	_lastMatched = _matched;
@@ -366,6 +306,16 @@ bool Macro::WasExecutedSince(const TimePoint &time) const
 	return _lastExecutionTime > time;
 }
 
+bool Macro::OnChangePreventedActionsSince(const TimePoint &time) const
+{
+	return _lastOnChangeActionsPreventedTime > time;
+}
+
+Macro::TimePoint Macro::GetLastExecutionTime() const
+{
+	return _lastExecutionTime;
+}
+
 bool Macro::ConditionsShouldBeChecked() const
 {
 	if (!_useCustomConditionCheckInterval) {
@@ -412,15 +362,9 @@ bool Macro::ShouldRunActions() const
 
 void Macro::SetName(const std::string &name)
 {
-	const bool nameChanged = _name == name;
 	_name = name;
-
 	SetHotkeysDesc();
-
-	if (nameChanged) {
-		_dockId = GenerateDockId();
-	}
-	EnableDock(_registerDock);
+	_dockSettings.HandleMacroNameChange();
 }
 
 void Macro::ResetTimers()
@@ -605,7 +549,7 @@ void Macro::SetInputVariables(const MacroInputVariables &inputVariables)
 	_inputVariables = inputVariables;
 }
 
-std::vector<TempVariable> Macro::GetTempVars(MacroSegment *filter) const
+std::vector<TempVariable> Macro::GetTempVars(const MacroSegment *filter) const
 {
 	std::vector<TempVariable> res;
 
@@ -633,7 +577,7 @@ std::vector<TempVariable> Macro::GetTempVars(MacroSegment *filter) const
 					    return ptr.get() == segment;
 				    }) != _conditions.end();
 	};
-	auto isAction = [this](MacroSegment *segment) -> bool {
+	auto isAction = [this](const MacroSegment *segment) -> bool {
 		return std::find_if(_actions.begin(), _actions.end(),
 				    [segment](
 					    const std::shared_ptr<MacroSegment>
@@ -641,7 +585,7 @@ std::vector<TempVariable> Macro::GetTempVars(MacroSegment *filter) const
 					    return ptr.get() == segment;
 				    }) != _actions.end();
 	};
-	auto isElseAction = [this](MacroSegment *segment) -> bool {
+	auto isElseAction = [this](const MacroSegment *segment) -> bool {
 		return std::find_if(_elseActions.begin(), _elseActions.end(),
 				    [segment](
 					    const std::shared_ptr<MacroSegment>
@@ -819,7 +763,7 @@ bool Macro::Save(obs_data_t *obj, bool saveForCopy) const
 			  _useCustomConditionCheckInterval);
 	_customConditionCheckInterval.Save(obj, "customConditionCheckInterval");
 
-	SaveDockSettings(obj, saveForCopy);
+	_dockSettings.Save(obj, saveForCopy);
 
 	SaveSplitterPos(_actionConditionSplitterPosition, obj,
 			"macroActionConditionSplitterPosition");
@@ -905,7 +849,7 @@ bool Macro::Load(obs_data_t *obj)
 		obs_data_get_bool(obj, "useCustomConditionCheckInterval");
 	_customConditionCheckInterval.Load(obj, "customConditionCheckInterval");
 
-	LoadDockSettings(obj);
+	_dockSettings.Load(obj);
 
 	LoadSplitterPos(_actionConditionSplitterPosition, obj,
 			"macroActionConditionSplitterPosition");
@@ -1018,12 +962,12 @@ bool Macro::PostLoad()
 bool Macro::SwitchesScene() const
 {
 	for (const auto &a : _actions) {
-		if (a->GetId() == GetSceneSwitchActionId()) {
+		if (a->GetId() == MacroAction::GetDefaultID()) {
 			return true;
 		}
 	}
 	for (const auto &a : _elseActions) {
-		if (a->GetId() == GetSceneSwitchActionId()) {
+		if (a->GetId() == MacroAction::GetDefaultID()) {
 			return true;
 		}
 	}
@@ -1056,18 +1000,8 @@ bool Macro::HasValidSplitterPositions() const
 	       !_elseActionSplitterPosition.empty();
 }
 
-bool Macro::OnChangePreventedActionsRecently()
-{
-	if (_onPreventedActionExecution) {
-		_onPreventedActionExecution = false;
-		return _matched ? _actions.size() > 0 : _elseActions.size() > 0;
-	}
-	return false;
-}
-
 void Macro::ResetUIHelpers()
 {
-	_onPreventedActionExecution = false;
 	for (auto c : _conditions) {
 		c->GetHighlightAndReset();
 	}
@@ -1095,224 +1029,6 @@ bool Macro::PauseHotkeysEnabled() const
 	return _registerHotkeys;
 }
 
-void Macro::SaveDockSettings(obs_data_t *obj, bool saveForCopy) const
-{
-	auto dockSettings = obs_data_create();
-	obs_data_set_bool(dockSettings, "register", _registerDock);
-	obs_data_set_bool(dockSettings, "hasRunButton", _dockHasRunButton);
-	obs_data_set_bool(dockSettings, "hasPauseButton", _dockHasPauseButton);
-	obs_data_set_bool(dockSettings, "hasStatusLabel", _dockHasStatusLabel);
-	obs_data_set_bool(dockSettings, "highlightIfConditionsTrue",
-			  _dockHighlight);
-	_runButtonText.Save(dockSettings, "runButtonText");
-	_pauseButtonText.Save(dockSettings, "pauseButtonText");
-	_unpauseButtonText.Save(dockSettings, "unpauseButtonText");
-	_conditionsTrueStatusText.Save(dockSettings,
-				       "conditionsTrueStatusText");
-	_conditionsFalseStatusText.Save(dockSettings,
-					"conditionsFalseStatusText");
-	if (saveForCopy) {
-		auto uuid = GenerateDockId();
-		obs_data_set_string(dockSettings, "dockId", uuid.c_str());
-
-	} else {
-		obs_data_set_string(dockSettings, "dockId", _dockId.c_str());
-	}
-	obs_data_set_int(dockSettings, "version", 1);
-	obs_data_set_obj(obj, "dockSettings", dockSettings);
-	obs_data_release(dockSettings);
-}
-
-void Macro::LoadDockSettings(obs_data_t *obj)
-{
-	auto dockSettings = obs_data_get_obj(obj, "dockSettings");
-	if (!dockSettings) {
-		// TODO: Remove this fallback
-		_dockHasRunButton = obs_data_get_bool(obj, "dockHasRunButton");
-		_dockHasPauseButton =
-			obs_data_get_bool(obj, "dockHasPauseButton");
-		EnableDock(obs_data_get_bool(obj, "registerDock"));
-		return;
-	}
-
-	if (!obs_data_has_user_value(dockSettings, "version")) {
-		_dockId = std::string("ADVSS-") + _name;
-	} else {
-		_dockId = obs_data_get_string(dockSettings, "dockId");
-	}
-
-	const bool dockEnabled = obs_data_get_bool(dockSettings, "register");
-
-	// TODO: remove these default settings in a future version
-	obs_data_set_default_string(
-		dockSettings, "runButtonText",
-		obs_module_text("AdvSceneSwitcher.macroDock.run"));
-	obs_data_set_default_string(
-		dockSettings, "pauseButtonText",
-		obs_module_text("AdvSceneSwitcher.macroDock.pause"));
-	obs_data_set_default_string(
-		dockSettings, "unpauseButtonText",
-		obs_module_text("AdvSceneSwitcher.macroDock.unpause"));
-	_runButtonText.Load(dockSettings, "runButtonText");
-	_pauseButtonText.Load(dockSettings, "pauseButtonText");
-	_unpauseButtonText.Load(dockSettings, "unpauseButtonText");
-	_conditionsTrueStatusText.Load(dockSettings,
-				       "conditionsTrueStatusText");
-	_conditionsFalseStatusText.Load(dockSettings,
-					"conditionsFalseStatusText");
-	if (dockEnabled) {
-		_dockHasRunButton =
-			obs_data_get_bool(dockSettings, "hasRunButton");
-		_dockHasPauseButton =
-			obs_data_get_bool(dockSettings, "hasPauseButton");
-		_dockHasStatusLabel =
-			obs_data_get_bool(dockSettings, "hasStatusLabel");
-		_dockHighlight = obs_data_get_bool(dockSettings,
-						   "highlightIfConditionsTrue");
-	}
-	EnableDock(dockEnabled);
-	obs_data_release(dockSettings);
-}
-
-void Macro::EnableDock(bool value)
-{
-	// Reset dock regardless
-	RemoveDock();
-
-	if (!value) {
-		_registerDock = value;
-		return;
-	}
-
-	_dock = new MacroDock(GetWeakMacroByName(_name.c_str()), _runButtonText,
-			      _pauseButtonText, _unpauseButtonText,
-			      _conditionsTrueStatusText,
-			      _conditionsFalseStatusText, _dockHighlight);
-
-	if (!obs_frontend_add_dock_by_id(_dockId.c_str(), _name.c_str(),
-					 _dock)) {
-		blog(LOG_INFO, "failed to add macro dock for macro %s",
-		     _name.c_str());
-		_dock->deleteLater();
-		_dock = nullptr;
-		_registerDock = false;
-		return;
-	}
-
-	_registerDock = value;
-}
-
-void Macro::SetDockHasRunButton(bool value)
-{
-	_dockHasRunButton = value;
-	if (!_dock) {
-		return;
-	}
-	_dock->ShowRunButton(value);
-}
-
-void Macro::SetDockHasPauseButton(bool value)
-{
-	_dockHasPauseButton = value;
-	if (!_dock) {
-		return;
-	}
-	_dock->ShowPauseButton(value);
-}
-
-void Macro::SetDockHasStatusLabel(bool value)
-{
-	_dockHasStatusLabel = value;
-	if (!_dock) {
-		return;
-	}
-	_dock->ShowStatusLabel(value);
-}
-
-void Macro::SetHighlightEnable(bool value)
-{
-	_dockHighlight = value;
-	if (!_dock) {
-		return;
-	}
-	_dock->EnableHighlight(value);
-}
-
-void Macro::SetRunButtonText(const std::string &text)
-{
-	_runButtonText = text;
-	if (!_dock) {
-		return;
-	}
-	_dock->SetRunButtonText(text);
-}
-
-void Macro::SetPauseButtonText(const std::string &text)
-{
-	_pauseButtonText = text;
-	if (!_dock) {
-		return;
-	}
-	_dock->SetPauseButtonText(text);
-}
-
-void Macro::SetUnpauseButtonText(const std::string &text)
-{
-	_unpauseButtonText = text;
-	if (!_dock) {
-		return;
-	}
-	_dock->SetUnpauseButtonText(text);
-}
-
-void Macro::SetConditionsTrueStatusText(const std::string &text)
-{
-	_conditionsTrueStatusText = text;
-	if (!_dock) {
-		return;
-	}
-	_dock->SetConditionsTrueText(text);
-}
-
-StringVariable Macro::ConditionsTrueStatusText() const
-{
-	return _conditionsTrueStatusText;
-}
-
-void Macro::SetConditionsFalseStatusText(const std::string &text)
-{
-	_conditionsFalseStatusText = text;
-	if (!_dock) {
-		return;
-	}
-	_dock->SetConditionsFalseText(text);
-}
-
-StringVariable Macro::ConditionsFalseStatusText() const
-{
-	return _conditionsFalseStatusText;
-}
-
-void Macro::RemoveDock()
-{
-	obs_frontend_remove_dock(_dockId.c_str());
-	_dock = nullptr;
-}
-
-std::string Macro::GenerateDockId()
-{
-#if LIBOBS_API_VER > MAKE_SEMANTIC_VERSION(30, 0, 0)
-	auto uuid = os_generate_uuid();
-	auto id = std::string("advss-macro-dock-") + std::string(uuid);
-	bfree(uuid);
-	return id;
-
-#else
-	static std::atomic_int16_t idCounter = 0;
-	return std::to_string(++idCounter);
-#endif
-}
-
 static void pauseCB(void *data, obs_hotkey_id, obs_hotkey_t *, bool pressed)
 {
 	if (pressed) {
@@ -1402,7 +1118,7 @@ void Macro::SetHotkeysDesc() const
 void SaveMacros(obs_data_t *obj)
 {
 	obs_data_array_t *macroArray = obs_data_array_create();
-	for (const auto &m : macros) {
+	for (const auto &m : GetTopLevelMacros()) {
 		obs_data_t *array_obj = obs_data_create();
 
 		m->Save(array_obj);
@@ -1416,7 +1132,9 @@ void SaveMacros(obs_data_t *obj)
 
 void LoadMacros(obs_data_t *obj)
 {
+	auto &macros = GetTopLevelMacros();
 	macros.clear();
+
 	obs_data_array_t *macroArray = obs_data_get_array(obj, "macros");
 	size_t count = obs_data_array_count(macroArray);
 
@@ -1466,15 +1184,10 @@ void LoadMacros(obs_data_t *obj)
 	}
 }
 
-std::deque<std::shared_ptr<Macro>> &GetMacros()
-{
-	return macros;
-}
-
 bool CheckMacros()
 {
 	bool matchFound = false;
-	for (const auto &m : macros) {
+	for (const auto &m : GetTopLevelMacros()) {
 		if (!m->ConditionsShouldBeChecked()) {
 			vblog(LOG_INFO,
 			      "skipping condition check for macro \"%s\" "
@@ -1501,7 +1214,7 @@ bool RunMacros()
 	// reordered while macros are currently being executed.
 	// For example, this can happen if a macro is performing a wait action,
 	// as the main lock will be unlocked during this time.
-	auto runPhaseMacros = macros;
+	auto runPhaseMacros = GetTopLevelMacros();
 
 	// Avoid deadlocks when opening settings window and calling frontend
 	// API functions at the same time.
@@ -1541,41 +1254,14 @@ bool RunMacros()
 
 void StopAllMacros()
 {
-	for (const auto &m : macros) {
+	for (const auto &m : GetAllMacros()) {
 		m->Stop();
 	}
 }
 
-Macro *GetMacroByName(const char *name)
-{
-	for (const auto &m : macros) {
-		if (m->Name() == name) {
-			return m.get();
-		}
-	}
-
-	return nullptr;
-}
-
-Macro *GetMacroByQString(const QString &name)
-{
-	return GetMacroByName(name.toUtf8().constData());
-}
-
-std::weak_ptr<Macro> GetWeakMacroByName(const char *name)
-{
-	for (const auto &m : macros) {
-		if (m->Name() == name) {
-			return m;
-		}
-	}
-
-	return {};
-}
-
 void InvalidateMacroTempVarValues()
 {
-	for (const auto &m : macros) {
+	for (const auto &m : GetTopLevelMacros()) {
 		// Do not invalidate the temp vars set during condition checks
 		// or action executions running in parallel to the "main" macro
 		// loop, as otherwise access to the information stored in those
@@ -1590,6 +1276,8 @@ void InvalidateMacroTempVarValues()
 
 std::shared_ptr<Macro> GetMacroWithInvalidConditionInterval()
 {
+	auto &macros = GetTopLevelMacros();
+
 	if (macros.empty()) {
 		return {};
 	}
