@@ -1,34 +1,42 @@
 #include "auto-update-tooltip-label.hpp"
 
+#include <QToolTip>
+
 namespace advss {
 
-AutoUpdateTooltipLabel::AutoUpdateTooltipLabel(
+AutoUpdateHelpIcon::AutoUpdateHelpIcon(
 	QWidget *parent, const std::function<QString()> &updateTooltipCallback,
 	int updateIntervalMs)
-	: QLabel(parent),
+	: HelpIcon("", parent),
 	  _callback(updateTooltipCallback),
 	  _timer(new QTimer(this)),
 	  _updateIntervalMs(updateIntervalMs)
 {
 	connect(_timer, &QTimer::timeout, this,
-		&AutoUpdateTooltipLabel::UpdateTooltip);
+		&AutoUpdateHelpIcon::UpdateTooltip);
 }
 
-void AutoUpdateTooltipLabel::enterEvent(QEnterEvent *event)
+void AutoUpdateHelpIcon::enterEvent(QEnterEvent *event)
 {
+	UpdateTooltip();
 	_timer->start(_updateIntervalMs);
 	QLabel::enterEvent(event);
 }
 
-void AutoUpdateTooltipLabel::leaveEvent(QEvent *event)
+void AutoUpdateHelpIcon::leaveEvent(QEvent *event)
 {
 	_timer->stop();
 	QLabel::leaveEvent(event);
 }
 
-void AutoUpdateTooltipLabel::UpdateTooltip()
+void AutoUpdateHelpIcon::UpdateTooltip()
 {
-	setToolTip(_callback());
+	if (!underMouse()) {
+		return;
+	}
+
+	const QString text = _callback();
+	QToolTip::showText(QCursor::pos(), text, this);
 }
 
 } // namespace advss
