@@ -1,14 +1,12 @@
 #include "non-modal-dialog.hpp"
 #include "obs-module-helper.hpp"
 
-#include <atomic>
 #include <mutex>
 #include <obs-frontend-api.h>
 #include <QDialogButtonBox>
 #include <QLabel>
 #include <QLayout>
 #include <QMainWindow>
-#include <QCoreApplication>
 
 namespace advss {
 
@@ -45,7 +43,10 @@ NonModalMessageDialog::NonModalMessageDialog(const QString &message, Type type,
 	setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
 	auto layout = new QVBoxLayout(this);
-	layout->addWidget(new QLabel(message, this));
+	auto label = new QLabel(message, this);
+	label->setTextInteractionFlags(Qt::TextBrowserInteraction);
+	label->setOpenExternalLinks(true);
+	layout->addWidget(label);
 
 	switch (type) {
 	case Type::INFO: {
