@@ -1,8 +1,14 @@
 #include "plugin-state-helpers.hpp"
+#include "macro-settings.hpp"
+#include "macro-signals.hpp"
 #include "switcher-data.hpp"
 
 namespace advss {
 
+static std::mutex initMutex;
+static std::mutex postLoadMutex;
+static std::mutex mutex;
+
 static std::vector<std::function<void()>> &getPluginInitSteps()
 {
 	static std::vector<std::function<void()>> steps;
@@ -39,7 +45,23 @@ static std::vector<std::function<void()>> &getStopSteps()
 	return steps;
 }
 
-static std::mutex mutex;
+static std::vector<std::function<void(obs_data_t *)>> &getSaveSteps()
+{
+	static std::vector<std::function<void(obs_data_t *)>> steps;
+	return steps;
+}
+
+static std::vector<std::function<void(obs_data_t *)>> &getLoadSteps()
+{
+	static std::vector<std::function<void(obs_data_t *)>> steps;
+	return steps;
+}
+
+static std::vector<std::function<void()>> &getPostLoadSteps()
+{
+	static std::vector<std::function<void()>> steps;
+	return steps;
+}
 
 void SavePluginSettings(obs_data_t *obj)
 {
@@ -53,17 +75,20 @@ void LoadPluginSettings(obs_data_t *obj)
 
 void AddSaveStep(std::function<void(obs_data_t *)> step)
 {
-	GetSwitcher()->AddSaveStep(step);
+	std::lock_guard<std::mutex> lock(mutex);
+	getSaveSteps().emplace_back(step);
 }
 
 void AddLoadStep(std::function<void(obs_data_t *)> step)
 {
-	GetSwitcher()->AddLoadStep(step);
+	std::lock_guard<std::mutex> lock(mutex);
+	getLoadSteps().emplace_back(step);
 }
 
 void AddPostLoadStep(std::function<void()> step)
 {
-	GetSwitcher()->AddPostLoadStep(step);
+	std::lock_guard<std::mutex> lock(postLoadMutex);
+	getPostLoadSteps().emplace_back(step);
 }
 
 void AddIntervalResetStep(std::function<void()> step)
@@ -72,14 +97,40 @@ void AddIntervalResetStep(std::function<void()> step)
 	getResetIntervalSteps().emplace_back(step);
 }
 
-void RunPostLoadSteps()
+void RunSaveSteps(obs_data_t *obj)
 {
-	GetSwitcher()->RunPostLoadSteps();
+	std::lock_guard<std::mutex> lock(mutex);
+	for (const auto &func : getSaveSteps()) {
+		func(obj);
+	}
+}
+
+void RunLoadSteps(obs_data_t *obj)
+{
+	std::lock_guard<std::mutex> lock(mutex);
+	for (const auto &func : getLoadSteps()) {
+		func(obj);
+	}
+}
+
+void RunAndClearPostLoadSteps()
+{
+	std::lock_guard<std::mutex> lock(postLoadMutex);
+	for (const auto &func : getPostLoadSteps()) {
+		func();
+	}
+	getPostLoadSteps().clear();
+}
+
+void ClearPostLoadSteps()
+{
+	std::lock_guard<std::mutex> lock(postLoadMutex);
+	getPostLoadSteps().clear();
 }
 
 void AddPluginInitStep(std::function<void()> step)
 {
-	std::lock_guard<std::mutex> lock(mutex);
+	std::lock_guard<std::mutex> lock(initMutex);
 	getPluginInitSteps().emplace_back(step);
 }
 
@@ -97,7 +148,7 @@ void AddPluginCleanupStep(std::function<void()> step)
 
 void RunPluginInitSteps()
 {
-	std::lock_guard<std::mutex> lock(mutex);
+	std::lock_guard<std::mutex> lock(initMutex);
 	for (const auto &step : getPluginInitSteps()) {
 		step();
 	}
@@ -188,7 +239,7 @@ NoMatchBehavior GetPluginNoMatchBehavior()
 
 void SetNoMatchScene(const OBSWeakSource &scene)
 {
-	GetSwitcher()->nonMatchingScene = scene;
+	GetSwitcher()->nonMatchingScene.SetScene(scene);
 }
 
 std::string ForegroundWindowTitle()
@@ -231,4 +282,30 @@ bool IsFirstIntervalAfterStop()
 	return GetSwitcher()->firstIntervalAfterStop;
 }
 
+void SetMacroHighlightingEnabled(bool enable)
+{
+	auto &settings = GetGlobalMacroSettings();
+	settings._highlightActions = enable;
+	settings._highlightConditions = enable;
+	settings._highlightExecuted = enable;
+
+	obs_queue_task(
+		OBS_TASK_UI,
+		[](void *) {
+			if (!SettingsWindowIsOpened()) {
+				return;
+			}
+			MacroSignalManager::Instance()->HighlightChanged(
+				GetGlobalMacroSettings()._highlightExecuted);
+		},
+		nullptr, false);
+}
+
+bool IsMacroHighlightingEnabled()
+{
+	const auto &settings = GetGlobalMacroSettings();
+	return settings._highlightActions || settings._highlightConditions ||
+	       settings._highlightExecuted;
+}
+
 } // namespace advss
