@@ -10,6 +10,8 @@ namespace advss {
 
 class StringList : public QList<StringVariable> {
 public:
+	using QList<StringVariable>::QList;
+
 	EXPORT bool Save(obs_data_t *obj, const char *name,
 			 const char *elementName = "string") const;
 	EXPORT bool Load(obs_data_t *obj, const char *name,
@@ -19,14 +21,20 @@ public:
 	friend class StringListEdit;
 };
 
-class ADVSS_EXPORT StringListEdit final : public ListEditor {
+class ADVSS_EXPORT StringListEdit : public ListEditor {
 	Q_OBJECT
 
 public:
-	StringListEdit(QWidget *parent, const QString &addString = "",
-		       const QString &addStringDescription = "",
-		       int maxStringSize = 170, bool allowEmtpy = false);
+	StringListEdit(
+		QWidget *parent, const QString &addString = "",
+		const QString &addStringDescription = "",
+		int maxStringSize = 170,
+		const std::function<bool(const std::string &)> &filter =
+			[](const std::string &) { return false; },
+		const std::function<void(std::string &input)> &preprocess =
+			[](std::string &) {});
 	void SetStringList(const StringList &);
+	StringList GetStringList() const { return _stringList; }
 	void SetMaxStringSize(int);
 
 private slots:
@@ -44,7 +52,8 @@ private:
 	QString _addString;
 	QString _addStringDescription;
 	int _maxStringSize = 170;
-	bool _allowEmpty = false;
+	std::function<bool(const std::string &)> _filterCallback;
+	std::function<void(std::string &)> _preprocessCallback;
 };
 
 } // namespace advss
