@@ -1,6 +1,7 @@
 #include "tab-helpers.hpp"
 #include "log-helper.hpp"
 #include "obs-module-helper.hpp"
+#include "plugin-state-helpers.hpp"
 
 #include <obs.hpp>
 #include <QTabBar>
@@ -17,7 +18,11 @@ static std::vector<std::string> tabNames = {
 	"videoTab",        "sceneGroupTab", "transitionsTab",   "pauseTab",
 };
 
-static std::vector<int> tabOrder = std::vector<int>(tabNames.size());
+static std::vector<int> &getTabOrderVector()
+{
+	static std::vector<int> tabOrder = std::vector<int>(tabNames.size());
+	return tabOrder;
+}
 
 namespace {
 struct TabCallbacks {
@@ -29,6 +34,21 @@ struct TabCallbacks {
 static std::unordered_map<const char *, TabCallbacks> createTabCallbacks;
 
 static int lastOpenedTab = -1;
+static bool alwaysShowTabs = false;
+
+[[maybe_unused]] static bool _ = []() {
+	AddPluginInitStep([]() {
+		AddSaveStep([](obs_data_t *data) {
+			obs_data_set_bool(data, "alwaysShowTabs",
+					  alwaysShowTabs);
+		});
+		AddLoadStep([](obs_data_t *data) {
+			alwaysShowTabs =
+				obs_data_get_bool(data, "alwaysShowTabs");
+		});
+	});
+	return true;
+}();
 
 void SetTabVisibleByName(QTabWidget *tabWidget, bool visible,
 			 const QString &name)
@@ -65,19 +85,14 @@ void ResetLastOpenedTab()
 	lastOpenedTab = -1;
 }
 
-void SaveTabOrder(obs_data_t *obj)
-{
-	OBSDataArrayAutoRelease tabWidgetOrder = obs_data_array_create();
-	for (size_t i = 0; i < tabNames.size(); i++) {
-		OBSDataAutoRelease entry = obs_data_create();
-		obs_data_set_int(entry, tabNames[i].c_str(), tabOrder[i]);
-		obs_data_array_push_back(tabWidgetOrder, entry);
-	}
-	obs_data_set_array(obj, "tabWidgetOrder", tabWidgetOrder);
-}
-
 static bool tabWidgetOrderValid()
 {
+	auto &tabOrder = getTabOrderVector();
+
+	if (tabNames.size() != tabOrder.size()) {
+		return false;
+	}
+
 	auto tmp = std::vector<int>(tabNames.size());
 	std::iota(tmp.begin(), tmp.end(), 0);
 
@@ -92,10 +107,29 @@ static bool tabWidgetOrderValid()
 
 static void resetTabWidgetOrder()
 {
+	auto &tabOrder = getTabOrderVector();
 	tabOrder = std::vector<int>(tabNames.size());
 	std::iota(tabOrder.begin(), tabOrder.end(), 0);
 }
 
+void SaveTabOrder(obs_data_t *obj)
+{
+	auto &tabOrder = getTabOrderVector();
+
+	// Can happen when corrupting settings files
+	if (!tabWidgetOrderValid()) {
+		resetTabWidgetOrder();
+	}
+
+	OBSDataArrayAutoRelease tabWidgetOrder = obs_data_array_create();
+	for (size_t i = 0; i < tabNames.size(); i++) {
+		OBSDataAutoRelease entry = obs_data_create();
+		obs_data_set_int(entry, tabNames[i].c_str(), tabOrder[i]);
+		obs_data_array_push_back(tabWidgetOrder, entry);
+	}
+	obs_data_set_array(obj, "tabWidgetOrder", tabWidgetOrder);
+}
+
 void LoadTabOrder(obs_data_t *obj)
 {
 	OBSDataArrayAutoRelease defaultTabWidgetOrder = obs_data_array_create();
@@ -107,6 +141,7 @@ void LoadTabOrder(obs_data_t *obj)
 	obs_data_set_default_array(obj, "tabWidgetOrder",
 				   defaultTabWidgetOrder);
 
+	auto &tabOrder = getTabOrderVector();
 	tabOrder.clear();
 	OBSDataArrayAutoRelease tabWidgetOrder =
 		obs_data_get_array(obj, "tabWidgetOrder");
@@ -144,6 +179,8 @@ void SetTabOrder(QTabWidget *tabWidget)
 		resetTabWidgetOrder();
 	}
 
+	auto &tabOrder = getTabOrderVector();
+
 	auto bar = tabWidget->tabBar();
 	for (int i = 0; i < bar->count(); ++i) {
 		int curPos = findTabIndex(tabWidget, tabOrder[i]);
@@ -153,9 +190,10 @@ void SetTabOrder(QTabWidget *tabWidget)
 		}
 	}
 
-	QWidget::connect(bar, &QTabBar::tabMoved, [](int from, int to) {
-		std::swap(tabOrder[from], tabOrder[to]);
-	});
+	QWidget::connect(bar, &QTabBar::tabMoved,
+			 [&tabOrder](int from, int to) {
+				 std::swap(tabOrder[from], tabOrder[to]);
+			 });
 }
 
 void SetCurrentTab(QTabWidget *tabWidget)
@@ -175,7 +213,41 @@ void SetupOtherTabs(QTabWidget *tabWidget)
 				.c_str());
 		tabWidget->insertTab(0, widget, tabText);
 		callbacks.setupTab(tabWidget);
+
+		if (alwaysShowTabs) {
+			tabWidget->setTabVisible(0, true);
+		}
 	}
 }
 
+void SetupShowAllTabsCheckBox(QCheckBox *checkBox, QTabWidget *tabWidget)
+{
+	const auto stateChanged = [tabWidget](int state) {
+		const bool newState = (state != 0);
+		if (newState == alwaysShowTabs) {
+			return;
+		}
+
+		alwaysShowTabs = newState;
+
+		// Only show currently hidden tabs, but don't hide tabs which
+		// are already visible
+		if (!alwaysShowTabs) {
+			return;
+		}
+
+		for (const auto &[name, _] : createTabCallbacks) {
+			const auto localeKey =
+				std::string("AdvSceneSwitcher.") + name +
+				".title";
+			auto tabText = obs_module_text(localeKey.c_str());
+			SetTabVisibleByName(tabWidget, true, tabText);
+		}
+	};
+
+	checkBox->setChecked(alwaysShowTabs);
+	QWidget::connect(checkBox, &QCheckBox::stateChanged, checkBox,
+			 stateChanged);
+}
+
 } // namespace advss
