@@ -1,5 +1,6 @@
 #include "macro-action-transition.hpp"
 #include "layout-helpers.hpp"
+#include "transition-helpers.hpp"
 
 #include <obs-frontend-api.h>
 
@@ -75,26 +76,6 @@ static void obs_sceneitem_set_transition_duration(obs_sceneitem_t *item,
 }
 #endif
 
-static void setSceneItemTransition(const OBSSceneItem &item,
-				   const OBSSourceAutoRelease &transition,
-				   bool show)
-{
-	OBSDataAutoRelease settings = obs_source_get_settings(transition);
-	if (!transition || !settings) {
-		// Set transition to "None"
-		obs_sceneitem_set_transition(item, show, nullptr);
-		return;
-	}
-
-	// We cannot share the transition source between
-	// scene items without introducing strange graphical
-	// artifacts so we have to create new ones here
-	OBSSourceAutoRelease transitionSource = obs_source_create_private(
-		obs_source_get_id(transition), obs_source_get_name(transition),
-		settings);
-	obs_sceneitem_set_transition(item, show, transitionSource);
-}
-
 void MacroActionTransition::SetSourceTransition(bool show)
 {
 #if LIBOBS_API_VER >= MAKE_SEMANTIC_VERSION(27, 0, 0)
@@ -103,7 +84,7 @@ void MacroActionTransition::SetSourceTransition(bool show)
 	const auto items = _source.GetSceneItems(_scene);
 	for (const auto &item : items) {
 		if (_setTransitionType) {
-			setSceneItemTransition(item, transition, show);
+			SetSceneItemTransition(item, transition, show);
 		}
 		if (_setDuration) {
 			obs_sceneitem_set_transition_duration(
@@ -237,7 +218,19 @@ MacroActionTransitionEdit::MacroActionTransitionEdit(
 	QWidget *parent, std::shared_ptr<MacroActionTransition> entryData)
 	: QWidget(parent),
 	  _actions(new QComboBox),
-	  _sources(new SceneItemSelectionWidget(parent, false)),
+	  _sources(new SceneItemSelectionWidget(
+		  parent,
+		  {
+			  SceneItemSelection::Type::SOURCE_NAME,
+			  SceneItemSelection::Type::VARIABLE_NAME,
+			  SceneItemSelection::Type::SOURCE_NAME_PATTERN,
+			  SceneItemSelection::Type::SOURCE_GROUP,
+			  SceneItemSelection::Type::SOURCE_TYPE,
+			  SceneItemSelection::Type::INDEX,
+			  SceneItemSelection::Type::INDEX_RANGE,
+			  SceneItemSelection::Type::ALL,
+		  },
+		  SceneItemSelectionWidget::NameClashMode::ALL)),
 	  _scenes(new SceneSelectionWidget(this, true, false, false, true)),
 	  _setTransition(new QCheckBox),
 	  _setDuration(new QCheckBox),
@@ -268,7 +261,7 @@ MacroActionTransitionEdit::MacroActionTransitionEdit(
 	QWidget::connect(_setDuration, SIGNAL(stateChanged(int)), this,
 			 SLOT(SetDurationChanged(int)));
 
-	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
+	const std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
 		{"{{type}}", _actions},
 		{"{{sources}}", _sources},
 		{"{{scenes}}", _scenes},
@@ -277,7 +270,8 @@ MacroActionTransitionEdit::MacroActionTransitionEdit(
 		{"{{setTransition}}", _setTransition},
 		{"{{setDuration}}", _setDuration},
 	};
-	QHBoxLayout *typeLayout = new QHBoxLayout;
+
+	auto typeLayout = new QHBoxLayout;
 	PlaceWidgets(obs_module_text(
 			     "AdvSceneSwitcher.action.transition.entry.line1"),
 		     typeLayout, widgetPlaceholders);
@@ -287,7 +281,7 @@ MacroActionTransitionEdit::MacroActionTransitionEdit(
 	PlaceWidgets(obs_module_text(
 			     "AdvSceneSwitcher.action.transition.entry.line3"),
 		     _durationLayout, widgetPlaceholders);
-	QVBoxLayout *mainLayout = new QVBoxLayout;
+	auto mainLayout = new QVBoxLayout;
 	mainLayout->addLayout(typeLayout);
 	mainLayout->addLayout(_transitionLayout);
 	mainLayout->addLayout(_durationLayout);
