@@ -17,6 +17,7 @@ bool MacroActionTwitch::_registered = MacroActionFactory::Register(
 void MacroActionTwitch::ResolveVariablesToFixedValues()
 {
 	_streamTitle.ResolveVariables();
+	_tags.ResolveVariables();
 	_markerDescription.ResolveVariables();
 	_duration.ResolveVariables();
 	_announcementMessage.ResolveVariables();
@@ -56,6 +57,12 @@ const static std::map<MacroActionTwitch::Action, std::string> actionTypes = {
 	 "AdvSceneSwitcher.action.twitch.type.channel.info.title.set"},
 	{MacroActionTwitch::Action::CHANNEL_INFO_CATEGORY_SET,
 	 "AdvSceneSwitcher.action.twitch.type.channel.info.category.set"},
+	{MacroActionTwitch::Action::CHANNEL_INFO_TAGS_SET,
+	 "AdvSceneSwitcher.action.twitch.type.channel.info.tags.set"},
+	{MacroActionTwitch::Action::CHANNEL_INFO_LANGUAGE_SET,
+	 "AdvSceneSwitcher.action.twitch.type.channel.info.language.set"},
+	{MacroActionTwitch::Action::CHANNEL_INFO_CONTENT_LABELS_SET,
+	 "AdvSceneSwitcher.action.twitch.type.channel.info.contentClassification.set"},
 	{MacroActionTwitch::Action::RAID_START,
 	 "AdvSceneSwitcher.action.twitch.type.raid.start"},
 	{MacroActionTwitch::Action::COMMERCIAL_START,
@@ -76,6 +83,8 @@ const static std::map<MacroActionTwitch::Action, std::string> actionTypes = {
 	 "AdvSceneSwitcher.action.twitch.type.user.getInfo"},
 	{MacroActionTwitch::Action::POINTS_REWARD_GET_INFO,
 	 "AdvSceneSwitcher.action.twitch.type.reward.getInfo"},
+	{MacroActionTwitch::Action::CHANNEL_GET_INFO,
+	 "AdvSceneSwitcher.action.twitch.type.channel.getInfo"},
 };
 
 const static std::map<MacroActionTwitch::AnnouncementColor, std::string>
@@ -315,6 +324,41 @@ void MacroActionTwitch::GetUserInfo(const std::shared_ptr<TwitchToken> &token)
 	});
 }
 
+void MacroActionTwitch::GetChannelInfo(const std::shared_ptr<TwitchToken> &token)
+{
+	auto info = _channel.GetInfo(*token);
+	if (!info) {
+		return;
+	}
+
+	SetTempVarValue("broadcaster_user_id", info->broadcaster_id);
+	SetTempVarValue("broadcaster_user_login", info->broadcaster_login);
+	SetTempVarValue("broadcaster_user_name", info->broadcaster_name);
+	SetTempVarValue("language", info->broadcaster_language);
+	SetTempVarValue("game_id", info->game_id);
+	SetTempVarValue("game_name", info->game_name);
+	SetTempVarValue("title", info->title);
+	SetTempVarValue("delay", std::to_string(info->delay));
+	std::string tags;
+	for (const auto &tag : info->tags) {
+		tags += tag + " ";
+	}
+	if (!tags.empty()) {
+		tags.pop_back();
+	}
+	SetTempVarValue("tags", tags);
+	std::string classificationLabels;
+	for (const auto &label : info->content_classification_labels) {
+		classificationLabels += label + " ";
+	}
+	if (!classificationLabels.empty()) {
+		classificationLabels.pop_back();
+	}
+	SetTempVarValue("content_classification_labels", classificationLabels);
+	SetTempVarValue("is_branded_content",
+			info->is_branded_content ? "true" : "false");
+}
+
 bool MacroActionTwitch::ResolveVariableSelectionToRewardId(
 	const std::shared_ptr<TwitchToken> &token)
 {
@@ -498,6 +542,16 @@ void MacroActionTwitch::SetupTempVars()
 		setupTempVarHelper("offline_image_url", ".user.getInfo");
 		setupTempVarHelper("created_at", ".user.getInfo");
 		break;
+	case Action::CHANNEL_GET_INFO:
+		setupTempVarHelper("language");
+		setupTempVarHelper("game_id");
+		setupTempVarHelper("game_name");
+		setupTempVarHelper("title");
+		setupTempVarHelper("delay");
+		setupTempVarHelper("tags");
+		setupTempVarHelper("content_classification_labels");
+		setupTempVarHelper("is_branded_content");
+		break;
 	default:
 		break;
 	}
@@ -517,6 +571,15 @@ bool MacroActionTwitch::PerformAction()
 	case Action::CHANNEL_INFO_CATEGORY_SET:
 		SetStreamCategory(token);
 		break;
+	case Action::CHANNEL_INFO_TAGS_SET:
+		_tags.SetStreamTags(*token);
+		break;
+	case Action::CHANNEL_INFO_LANGUAGE_SET:
+		_language.SetStreamLanguage(*token);
+		break;
+	case Action::CHANNEL_INFO_CONTENT_LABELS_SET:
+		_contentClassification.SetContentClassification(*token);
+		break;
 	case Action::RAID_START:
 		StartRaid(token);
 		break;
@@ -547,6 +610,9 @@ bool MacroActionTwitch::PerformAction()
 	case MacroActionTwitch::Action::POINTS_REWARD_GET_INFO:
 		GetRewardInfo(token);
 		break;
+	case MacroActionTwitch::Action::CHANNEL_GET_INFO:
+		GetChannelInfo(token);
+		break;
 	default:
 		break;
 	}
@@ -576,6 +642,9 @@ bool MacroActionTwitch::Save(obs_data_t *obj) const
 			    GetWeakTwitchTokenName(_token).c_str());
 	_streamTitle.Save(obj, "streamTitle");
 	_category.Save(obj);
+	_tags.Save(obj);
+	_language.Save(obj);
+	_contentClassification.Save(obj);
 	_markerDescription.Save(obj, "markerDescription");
 	obs_data_set_bool(obj, "clipHasDelay", _clipHasDelay);
 	_duration.Save(obj);
@@ -604,6 +673,9 @@ bool MacroActionTwitch::Load(obs_data_t *obj)
 	_token = GetWeakTwitchTokenByName(obs_data_get_string(obj, "token"));
 	_streamTitle.Load(obj, "streamTitle");
 	_category.Load(obj);
+	_tags.Load(obj);
+	_language.Load(obj);
+	_contentClassification.Load(obj);
 	_markerDescription.Load(obj, "markerDescription");
 	_clipHasDelay = obs_data_get_bool(obj, "clipHasDelay");
 	_duration.Load(obj);
@@ -642,6 +714,10 @@ bool MacroActionTwitch::ActionIsSupportedByToken()
 			 {{"channel:manage:broadcast"}}},
 			{Action::CHANNEL_INFO_CATEGORY_SET,
 			 {{"channel:manage:broadcast"}}},
+			{Action::CHANNEL_INFO_TAGS_SET,
+			 {{"channel:manage:broadcast"}}},
+			{Action::CHANNEL_INFO_CONTENT_LABELS_SET,
+			 {{"channel:manage:broadcast"}}},
 			{Action::CHANNEL_INFO_LANGUAGE_SET,
 			 {{"channel:manage:broadcast"}}},
 			{Action::CHANNEL_INFO_DELAY_SET,
@@ -746,6 +822,7 @@ bool MacroActionTwitch::ActionIsSupportedByToken()
 			{Action::WHISPER_SEND, {{"user:manage:whispers"}}},
 			{Action::SEND_CHAT_MESSAGE, {{"chat:edit"}}},
 			{Action::USER_GET_INFO, {}},
+			{Action::CHANNEL_GET_INFO, {}},
 			{Action::POINTS_REWARD_GET_INFO,
 			 {{"channel:read:redemptions"},
 			  {"channel:manage:redemptions"}}}};
@@ -815,6 +892,9 @@ MacroActionTwitchEdit::MacroActionTwitchEdit(
 	  _tokenWarning(new QLabel()),
 	  _streamTitle(new VariableLineEdit(this)),
 	  _category(new TwitchCategoryWidget(this)),
+	  _tags(new TagListWidget(this)),
+	  _language(new LanguageSelectionWidget(this)),
+	  _contentClassification(new ContentClassificationEdit(this)),
 	  _markerDescription(new VariableLineEdit(this)),
 	  _clipHasDelay(new QCheckBox(obs_module_text(
 		  "AdvSceneSwitcher.action.twitch.clip.hasDelay"))),
@@ -840,6 +920,9 @@ MacroActionTwitchEdit::MacroActionTwitchEdit(
 	mainLayout->addLayout(_layout);
 	mainLayout->addWidget(_announcementMessage);
 	mainLayout->addWidget(_chatMessage);
+	mainLayout->addWidget(_tags);
+	mainLayout->addWidget(_language);
+	mainLayout->addWidget(_contentClassification);
 	mainLayout->addWidget(_tokenWarning);
 	setLayout(mainLayout);
 
@@ -876,6 +959,9 @@ void MacroActionTwitchEdit::TwitchTokenChanged(const QString &token)
 	_category->SetToken(_entryData->_token);
 	_channel->SetToken(_entryData->_token);
 	_pointsReward->SetToken(_entryData->_token);
+	_tags->SetToken(_entryData->_token);
+	_language->SetToken(_entryData->_token);
+	_contentClassification->SetToken(_entryData->_token);
 	_entryData->ResetChatConnection();
 
 	SetWidgetVisibility();
@@ -930,6 +1016,25 @@ void MacroActionTwitchEdit::CategoryChanged(const TwitchCategory &category)
 	_entryData->_category = category;
 }
 
+void MacroActionTwitchEdit::TagsChanged(const TwitchTagList &tags)
+{
+	GUARD_LOADING_AND_LOCK();
+	_entryData->_tags = tags;
+}
+
+void MacroActionTwitchEdit::LanguageChanged(const LanguageSelection &language)
+{
+	GUARD_LOADING_AND_LOCK();
+	_entryData->_language = language;
+}
+
+void MacroActionTwitchEdit::ContentClassificationChanged(
+	const ContentClassification &ccl)
+{
+	GUARD_LOADING_AND_LOCK();
+	_entryData->_contentClassification = ccl;
+}
+
 void MacroActionTwitchEdit::MarkerDescriptionChanged()
 {
 	GUARD_LOADING_AND_LOCK();
@@ -1010,6 +1115,18 @@ void MacroActionTwitchEdit::SetWidgetSignalConnections()
 	QWidget::connect(_category,
 			 SIGNAL(CategoryChanged(const TwitchCategory &)), this,
 			 SLOT(CategoryChanged(const TwitchCategory &)));
+	QWidget::connect(_tags, SIGNAL(TagListChanged(const TwitchTagList &)),
+			 this, SLOT(TagsChanged(const TwitchTagList &)));
+	QWidget::connect(_language,
+			 SIGNAL(LanguageChanged(const LanguageSelection &)),
+			 this,
+			 SLOT(LanguageChanged(const LanguageSelection &)));
+	QWidget::connect(_contentClassification,
+			 SIGNAL(ContentClassificationChanged(
+				 const ContentClassification &)),
+			 this,
+			 SLOT(ContentClassificationChanged(
+				 const ContentClassification &)));
 	QWidget::connect(_markerDescription, SIGNAL(editingFinished()), this,
 			 SLOT(MarkerDescriptionChanged()));
 	QObject::connect(_clipHasDelay, SIGNAL(stateChanged(int)), this,
@@ -1052,7 +1169,17 @@ void MacroActionTwitchEdit::SetWidgetVisibility()
 	_category->setVisible(
 		_entryData->GetAction() ==
 		MacroActionTwitch::Action::CHANNEL_INFO_CATEGORY_SET);
+	_tags->setVisible(_entryData->GetAction() ==
+			  MacroActionTwitch::Action::CHANNEL_INFO_TAGS_SET);
+	_language->setVisible(
+		_entryData->GetAction() ==
+		MacroActionTwitch::Action::CHANNEL_INFO_LANGUAGE_SET);
+	_contentClassification->setVisible(
+		_entryData->GetAction() ==
+		MacroActionTwitch::Action::CHANNEL_INFO_CONTENT_LABELS_SET);
 	_channel->setVisible(
+		_entryData->GetAction() ==
+			MacroActionTwitch::Action::CHANNEL_GET_INFO ||
 		_entryData->GetAction() ==
 			MacroActionTwitch::Action::RAID_START ||
 		_entryData->GetAction() ==
@@ -1199,19 +1326,23 @@ void MacroActionTwitchEdit::SetWidgetLayout()
 	switch (_entryData->GetAction()) {
 	case MacroActionTwitch::Action::SEND_CHAT_MESSAGE:
 		layoutText = obs_module_text(
-			"AdvSceneSwitcher.action.twitch.entry.chat");
+			"AdvSceneSwitcher.action.twitch.layout.chat");
 		break;
 	case MacroActionTwitch::Action::USER_GET_INFO:
 		layoutText = obs_module_text(
-			"AdvSceneSwitcher.action.twitch.entry.user.getInfo");
+			"AdvSceneSwitcher.action.twitch.layout.user.getInfo");
 		break;
 	case MacroActionTwitch::Action::POINTS_REWARD_GET_INFO:
 		layoutText = obs_module_text(
-			"AdvSceneSwitcher.action.twitch.entry.reward.getInfo");
+			"AdvSceneSwitcher.action.twitch.layout.reward.getInfo");
+		break;
+	case MacroActionTwitch::Action::CHANNEL_GET_INFO:
+		layoutText = obs_module_text(
+			"AdvSceneSwitcher.action.twitch.layout.channel.getInfo");
 		break;
 	default:
 		layoutText = obs_module_text(
-			"AdvSceneSwitcher.action.twitch.entry.default");
+			"AdvSceneSwitcher.action.twitch.layout.default");
 		break;
 	}
 
@@ -1247,6 +1378,13 @@ void MacroActionTwitchEdit::UpdateEntryData()
 	_streamTitle->setText(_entryData->_streamTitle);
 	_category->SetToken(_entryData->_token);
 	_category->SetCategory(_entryData->_category);
+	_tags->SetTags(_entryData->_tags);
+	_tags->SetToken(_entryData->_token);
+	_language->SetLanguageSelection(_entryData->_language);
+	_language->SetToken(_entryData->_token);
+	_contentClassification->SetContentClassification(
+		_entryData->_contentClassification);
+	_contentClassification->SetToken(_entryData->_token);
 	_markerDescription->setText(_entryData->_markerDescription);
 	_clipHasDelay->setChecked(_entryData->_clipHasDelay);
 	_duration->SetDuration(_entryData->_duration);
