@@ -1,6 +1,7 @@
 #include "token.hpp"
 #include "twitch-helpers.hpp"
 
+#include <help-icon.hpp>
 #include <layout-helpers.hpp>
 #include <log-helper.hpp>
 #include <obs-module-helper.hpp>
@@ -31,6 +32,7 @@ const std::unordered_map<std::string, std::string> TokenOption::_apiIdToLocale{
 	 "AdvSceneSwitcher.twitchToken.channel.raids.manage"},
 	{"moderator:manage:shoutouts",
 	 "AdvSceneSwitcher.twitchToken.moderator.shoutouts.manage"},
+	{"channel:read:ads", "AdvSceneSwitcher.twitchToken.channel.ads.read"},
 	{"channel:read:polls",
 	 "AdvSceneSwitcher.twitchToken.channel.polls.read"},
 	{"channel:manage:polls",
@@ -162,7 +164,8 @@ TwitchToken::TwitchToken(const TwitchToken &other)
 	  _userID(other._userID),
 	  _tokenOptions(other._tokenOptions),
 	  _eventSub(),
-	  _validateEventSubTimestamps(other._validateEventSubTimestamps)
+	  _validateEventSubTimestamps(other._validateEventSubTimestamps),
+	  _warnIfInvalid(other._warnIfInvalid)
 {
 }
 
@@ -173,6 +176,7 @@ TwitchToken &TwitchToken::operator=(const TwitchToken &other)
 	_userID = other._userID;
 	_tokenOptions = other._tokenOptions;
 	_validateEventSubTimestamps = other._validateEventSubTimestamps;
+	_warnIfInvalid = other._warnIfInvalid;
 	return *this;
 }
 
@@ -184,6 +188,9 @@ void TwitchToken::Load(obs_data_t *obj)
 	obs_data_set_default_bool(obj, "validateEventSubTimestamps", true);
 	_validateEventSubTimestamps =
 		obs_data_get_bool(obj, "validateEventSubTimestamps");
+	_warnIfInvalid = obs_data_has_user_value(obj, "warnIfInvalid")
+				 ? obs_data_get_bool(obj, "warnIfInvalid")
+				 : true;
 	_tokenOptions.clear();
 	OBSDataArrayAutoRelease options = obs_data_get_array(obj, "options");
 	size_t count = obs_data_array_count(options);
@@ -202,6 +209,7 @@ void TwitchToken::Save(obs_data_t *obj) const
 	obs_data_set_string(obj, "userID", _userID.c_str());
 	obs_data_set_bool(obj, "validateEventSubTimestamps",
 			  _validateEventSubTimestamps);
+	obs_data_set_bool(obj, "warnIfInvalid", _warnIfInvalid);
 	OBSDataArrayAutoRelease options = obs_data_array_create();
 	for (auto &option : _tokenOptions) {
 		OBSDataAutoRelease arrayObj = obs_data_create();
@@ -476,7 +484,9 @@ TwitchTokenSettingsDialog::TwitchTokenSettingsDialog(
 	  _tokenStatus(new QLabel()),
 	  _generalSettingsGrid(new QGridLayout()),
 	  _validateTimestamps(new QCheckBox(obs_module_text(
-		  "AdvSceneSwitcher.twitchToken.validateTimestamps")))
+		  "AdvSceneSwitcher.twitchToken.validateTimestamps"))),
+	  _warnIfInvalid(new QCheckBox(obs_module_text(
+		  "AdvSceneSwitcher.twitchToken.warnIfInvalid")))
 {
 	_showToken->setMaximumWidth(22);
 	_showToken->setFlat(true);
@@ -538,11 +548,18 @@ TwitchTokenSettingsDialog::TwitchTokenSettingsDialog(
 	scrollArea->setWidgetResizable(true);
 	scrollArea->setFrameShape(QFrame::NoFrame);
 
+	auto timestampsLayout = new QHBoxLayout();
+	timestampsLayout->addWidget(_validateTimestamps);
+	auto timestampHelp = new HelpIcon(obs_module_text(
+		"AdvSceneSwitcher.twitchToken.validateTimestamps.tooltip"));
+	timestampsLayout->addWidget(timestampHelp);
+
 	auto contentWidget = new QWidget(scrollArea);
 	auto layout = new QVBoxLayout(contentWidget);
 	layout->addLayout(_generalSettingsGrid);
 	layout->addWidget(optionsBox);
-	layout->addWidget(_validateTimestamps);
+	layout->addLayout(timestampsLayout);
+	layout->addWidget(_warnIfInvalid);
 	layout->setContentsMargins(0, 0, 0, 0);
 	scrollArea->setWidget(contentWidget);
 
@@ -568,6 +585,7 @@ TwitchTokenSettingsDialog::TwitchTokenSettingsDialog(
 #ifndef VERIFY_TIMESTAMPS
 	_validateTimestamps->hide();
 #endif
+	_warnIfInvalid->setChecked(settings._warnIfInvalid);
 
 	_currentToken = settings;
 
@@ -776,6 +794,8 @@ bool TwitchTokenSettingsDialog::AskForSettings(QWidget *parent,
 	settings._tokenOptions = dialog.GetEnabledOptions();
 	settings._validateEventSubTimestamps =
 		dialog._validateTimestamps->isChecked();
+	settings._warnIfInvalid = dialog._warnIfInvalid->isChecked();
+
 	if (settings._eventSub) {
 		settings._eventSub->EnableTimestampValidation(
 			settings._validateEventSubTimestamps);
@@ -809,7 +829,7 @@ void TokenGrabberThread::run()
 	if (_serverThread.joinable()) {
 		_serverThread.join();
 	}
-	_stopWaiting = {false};
+	_stopWaiting = false;
 
 	// Generate URI to request token
 	auto state = generateStateString();
